# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, you can obtain one at http://mozilla.org/MPL/2.0/.
import logging
import os

import requests
from flask import render_template_string
from redash.handlers.authentication import base_href
from redash.handlers.base import BaseResource, get_object_or_404
from redash.models import Query
from redash.permissions import require_permission
from redash_iodide import settings
from redash_iodide.resources import add_resource

logger = logging.getLogger(__name__)


class IodideNotebookResource(BaseResource):
    TEMPLATE_PATH = os.path.join(os.path.dirname(__file__), "iodide-notebook.iomd.j2")

    @require_permission("view_query")
    def post(self, query_id):
        query = get_object_or_404(Query.get_by_id_and_org, query_id, self.current_org)

        with open(self.TEMPLATE_PATH, "r") as template:
            source = template.read()
            context = {
                "redash_url": base_href(),
                "query_id": query_id,
                "title": query.name,
                "api_key": query.api_key,
            }
            rendered_template = render_template_string(source, **context)
        headers = {"Authorization": "Token %s" % settings.IODIDE_AUTH_TOKEN}
        data = {
            "owner": self.current_user.email,
            "title": query.name,
            "content": rendered_template,
        }
        response = requests.post(
            settings.IODIDE_NOTEBOOK_API_URL, headers=headers, data=data
        )
        return response.json()


def extension(app):
    logger.info("Loading Iodide integration extension")
    add_resource(
        app,
        IodideNotebookResource,
        "/api/integrations/iodide/<query_id>/notebook",
        endpoint="idodide_notebook",
    )
    logger.info("Loaded Iodide integration extension")



from setuptools import setup, find_packages
import os.path

readme = ""
here = os.path.abspath(os.path.dirname(__file__))
readme_path = os.path.join(here, "README.rst")
if os.path.exists(readme_path):
    with open(readme_path, "rb") as stream:
        readme = stream.read().decode("utf8")


setup(
    long_description=readme,
    long_description_content_type="text/x-rst",
    name="redash-iodide",
    use_scm_version={
        'version_scheme': 'post-release',
        'local_scheme': 'dirty-tag'
    },
    setup_requires=['setuptools_scm'],
    description="Extensions to Redash by Mozilla",
    python_requires="==2.*,>=2.7.0",
    project_urls={"homepage": "https://github.com/iodide-project/redash-iodide"},
    author="Mozilla Foundation",
    author_email="dev-webdev@lists.mozilla.org",
    license="MPL-2.0",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment :: Mozilla",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Internet :: WWW/HTTP",
    ],
    entry_points={
        "redash.extensions": [
            "iodide_explore = redash_iodide.explore.extension:extension",
            "iodide_settings = redash_iodide.settings:extension",
        ],
        "redash.bundles": [
            "iodide_explore = redash_iodide.explore",
        ],
        "redash.periodic_tasks": [
        ],
    },
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    install_requires=[
        "requests",
        "six",
        "PyJWT",
    ],
    extras_require={
        "test": [
            "flake8==3.5.0",
            "mock",
            "pytest",
            "pytest-cov",
            "pytest-flake8<1.0.1",
        ],
        "dev": ["watchdog"],
    },
)



import json
import os

import mock
from mock import patch
from redash_iodide import settings
from six.moves import reload_module
from tests import BaseTestCase, authenticate_request


class TestIodideIntegration(BaseTestCase):
    SETTING_OVERRIDES = {"REDASH_IODIDE_URL": "https://example.com/"}

    def setUp(self):
        super(TestIodideIntegration, self).setUp()
        self.admin = self.factory.create_admin()
        self.data_source = self.factory.create_data_source()
        variables = self.SETTING_OVERRIDES.copy()
        with patch.dict(os.environ, variables):
            reload_module(settings)

        # Queue a cleanup routine that reloads the settings without overrides
        # once the test ends
        self.addCleanup(lambda: reload_module(settings))
        authenticate_request(self.client, self.admin)

    def test_settings(self):
        rv = self.client.get("/api/integrations/iodide/settings",)
        self.assertEquals(rv.status_code, 200)
        self.assertEquals(
            rv.data,
            json.dumps({"iodideURL": self.SETTING_OVERRIDES["REDASH_IODIDE_URL"]}),
        )

    @mock.patch("requests.post")
    def test_notebook_post(self, mock_post):
        query = self.factory.create_query(
            user=self.admin,
            data_source=self.data_source,
            query_text="select * from events",
        )

        mock_value = {"id": query.id}
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.text = json.dumps(mock_value)
        mock_json = mock.Mock()
        mock_json.return_value = mock_value
        mock_response.json = mock_json
        mock_post.return_value = mock_response

        rv = self.client.post("/api/integrations/iodide/%s/notebook" % query.id)
        self.assertEquals(rv.status_code, 200)
        self.assertEquals(rv.data, json.dumps({"id": query.id}))



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, you can obtain one at http://mozilla.org/MPL/2.0/.
import logging
import os

from redash.handlers.base import BaseResource
from redash.permissions import require_permission

from .resources import add_resource

logger = logging.getLogger(__name__)


# The base URL of Mozilla's private Iodide instance
IODIDE_URL = os.environ.get("REDASH_IODIDE_URL", "")

# The Iodide API endpoint to hit to create a new notebook
IODIDE_NOTEBOOK_API_URL = os.environ.get("REDASH_IODIDE_NOTEBOOK_API_URL", "")

# The auth token that this extension uses to create new Iodide notebooks
IODIDE_AUTH_TOKEN = os.environ.get("REDASH_IODIDE_AUTH_TOKEN", "")


class IodideSettingsResource(BaseResource):
    @require_permission("view_query")
    def get(self):
        return {"iodideURL": IODIDE_URL}


def extension(app):
    logger.info("Loading Iodide setting extension")
    add_resource(
        app,
        IodideSettingsResource,
        "/api/integrations/iodide/settings",
        endpoint="idodide_settings",
    )
    logger.info("Loaded Iodide setting extension")
